/**
 * <copyright>
 * IBM WebSphere Studio Application Developer XML Tools
 * (C) Copyright IBM Corp. 2001.
 * </copyright>
 */
package com.ibm.etools.xmlschema.beans;

import org.w3c.dom.*;
import java.util.*;
import java.text.SimpleDateFormat;

public class BaseType implements java.io.Serializable
{
  protected Factory factory;
  protected Element xmlElement;
  protected Attr xmlAttr;

  protected SimpleDateFormat simpleDateFormat;

  public BaseType()
  {   
    simpleDateFormat = new SimpleDateFormat("yyyy-MM-dd");    
  }

  /**
   * Remember the factory that creates this XML element or attribute
   */
  protected void setFactory(Factory factory)
  {
    this.factory = factory;
  }

  /**
   * Return the factory
   */
  protected Factory getFactory()
  {
    return factory;
  }

  /**
   * Get local element prefix
   */
  public String getPrefix()
  {
    return getFactory().getPrefix();
  }

  /**
   * Indicates if local element needs to be qualifed or not
   */
  public boolean isElementQualified()
  {
    return getFactory().isElementQualified();
  }

  /**
   * Indicates if local attribute needs to be qualifed or not
   */
  public boolean isAttributeQualified()
  {
    return getFactory().isAttributeQualified();
  }

  /**
   * Remember the XML element that corresponds to this bean
   */
  protected void setXMLElement(Element element)
  {
    xmlElement = element;
  }

  /** 
   * Return the XML element that corresponds to this bean
   */
  public Element getXMLElement()
  {
    return xmlElement;
  }

  /**
   * Remember the XML attribute that corresponds to this bean
   */
  protected void setXMLAttribute(Attr xmlAttr)
  {
    this.xmlAttr = xmlAttr;
  }

  /**
   * Return the XML attribute that corresponds to this bean
   */
  public Attr getXMLAttribute()
  {
    return xmlAttr;
  }

  /**
   * Create the element and the text node if it does not exist.
   * Otherwise update the text value.
   *  @param elementName - the name of the element
   *  @param elementValue - the value of the element
   */
  public void setElementValue(String elementName, String elementValue)
  {         
    String elementPrefix = "";
    if (isElementQualified()) 
    {
      elementPrefix = getPrefix();
    }

    NodeList nodes = xmlElement.getElementsByTagName(elementPrefix + elementName);

    if (nodes.getLength() == 0) 
    {
      // Create a new element and text node and add 
      // that as a child to the current XML element
      Element elementNode = factory.createXMLElementAndText(elementName, elementValue);
      xmlElement.appendChild(elementNode);
    }
    else
    { 
      for (int i=0; i < nodes.getLength(); i++)
      {
        Node node = nodes.item(i);
        String item = node.getNodeName().trim();

        if (item.equals(elementPrefix + elementName))
        {
          xmlElement.removeChild(node);
          Element elementNode = factory.createXMLElementAndText(elementName, elementValue);
          xmlElement.appendChild(elementNode);
          break;
        }
      }
    }
  }

  /**
   * Create an element and set it to the specified value
   */
  public void setElementValue(String elementName, boolean value)
  {
    setElementValue(elementName, (new Boolean(value)).toString());
  }

  /**
   * Create an element and set it to the specified value
   */
  public void setElementValue(String elementName, int value)
  {
    setElementValue(elementName, (new Integer(value)).toString());
  }

  /**
   * Create an element and set it to the specified value
   */
  public void setElementValue(String elementName, float value)
  {
    setElementValue(elementName, (new Float(value)).toString());
  }

  /**
   * Create an element and set it to the specified value
   */
  public void setElementValue(String elementName, double value)
  {
    setElementValue(elementName, (new Double(value)).toString());
  }

  /**
   * Create an element and set it to the specified value
   */
  public void setElementValue(String elementName, long value)
  {
    setElementValue(elementName, (new Long(value)).toString());
  }

  /**
   * Create an element and set it to the specified value
   */
  public void setElementValue(String elementName, short value)
  {
    setElementValue(elementName, (new Short(value)).toString());
  }

  /**
   * Create an element and set it to the specified value
   */
  public void setElementValue(String elementName, byte value)
  {   
    setElementValue(elementName, (new Byte(value)).toString());
  }

  /**
   * Create an element and set it to the specified value. 
   * The Date string is converted into the XML Schema date format CCYY-MM-DD.
   */
  public void setElementValue(String elementName, Date value)
  {  
    setElementValue(elementName, simpleDateFormat.format(value));
  }

  /**
   * Create an element and set it to the specified value
   */
  public void setElementValue(String elementName, BaseType baseType)
  {
    Element childXml = baseType.getXMLElement();
    this.xmlElement.appendChild(childXml);
  }

  /**
   * Get the text value for this element
   */
  public String getElementValue()
  {
    // return node value
    if (xmlAttr != null) 
    {
      return  xmlAttr.getValue();
    }

    String value = xmlElement.getNodeValue();
    if (value!=null)
    {
      return value;
    }

    // fails, return PCDATA
    NodeList nList = xmlElement.getChildNodes();

    for (int i=0; i<nList.getLength(); i++)
    {
      if (nList.item(i) instanceof Text)
        return ((Text)nList.item(i)).getData();
    }
    return null;
  }

  /**
   * Get the element with the specifed name and at the specified location
   */
  public String getElementValue(String property, int index)
  { 
    String elementPrefix = "";
    if (isElementQualified()) 
    {
      elementPrefix = getPrefix();
    }

    NodeList nodes = xmlElement.getElementsByTagName(elementPrefix + property);
    for (int i=0; i < nodes.getLength(); i++)
    {
      Node node = nodes.item(i);
      String item = node.getNodeName().trim();

      if (item.equals(elementPrefix + property) && i == index)
      {
        Element element = (Element) node;
        Node child = element.getFirstChild();
        if (child instanceof Text) 
        {
          return ((Text)child).getData();
        }
      }
    }
    return null;
  }

  /**
   * Get the value for the input element
   *  @param elementName The name of the element
   *  @return String The value of the element (from the text node)
   */
  public String getElementValue(String elementName)
  {
    if (isElementQualified()) 
    {
      elementName = getPrefix() + elementName;
    }

    NodeList nList = xmlElement.getChildNodes();
    for (int i=0; i< nList.getLength(); i++)
    {
      Node node = nList.item(i);

      // Found the node that matches the element name
      if ( node.getNodeName().equals(elementName) )
      {
        if (node.getNodeType() == Node.ELEMENT_NODE)
        {
          Element anElement = (Element) node;
          Node child = anElement.getFirstChild();
          StringBuffer result = new StringBuffer("");
          while (child != null)
          {
            if (child.getNodeType() == Node.CDATA_SECTION_NODE)
            {
          	  result.append("<![CDATA[");
          	  result.append(((CDATASection)child).getData());
          	  result.append("]]>");
            }
            else if (child instanceof Text) 
            {
              result.append(((Text)child).getData());
            }
            child = child.getNextSibling();
          } 
          return result.toString();         
        }
      }
    }
    return null;
  }

  public boolean getElementBooleanValue(String elementName)
  { 
    return new Boolean(getElementValue(elementName)).booleanValue();
  }

  public int getElementIntegerValue(String elementName)
  { 
    return new Integer(getElementValue(elementName)).intValue();
  }

  public float getElementFloatValue(String elementName)
  {  
    return new Float(getElementValue(elementName)).floatValue();
  }

  public double getElementDoubleValue(String elementName)
  {
    return new Double(getElementValue(elementName)).doubleValue();
  }

  public long getElementLongValue(String elementName)
  { 
    return new Long(getElementValue(elementName)).longValue();
  }

  public short getElementShortValue(String elementName)
  { 
    return new Short(getElementValue(elementName)).shortValue();
  }

  public byte getElementByteValue(String elementName)
  { 
    return new Byte(getElementValue(elementName)).byteValue();   
  }

  /** 
   * Get the value for the input element
   *  @param elementName The name of the element
   *  @return Date The Java Date for the element value if the value conforms to the
   *               pattern yyyy-mm-dd
   */
  public Date getElementDateValue(String elementName)
  { 
    try
    {
      String result = getElementValue(elementName);
      if (result != null)
      {
        return simpleDateFormat.parse(result);
      }
    }
    catch (java.text.ParseException ex)
    {
      System.out.println("getElementDateValue exception.." + ex);
    }
    return null;
  }

  /**
   * Get the value for the input element
   * @param className - the name of the class corresponding to this element
   *                    Could be an inner class.
   */
  public BaseType getElementValue(String property, String className)
  {
    if (isElementQualified()) 
    {
      property = getPrefix() + property;
    }

    NodeList nList = xmlElement.getChildNodes();

    for ( int i=0; i < nList.getLength(); i++ )
    {
      Node node = nList.item(i);

      if ( node.getNodeName().equals(property) )
      {
        // ressurect this object
        return factory.newInstance((Element) node, className);
      }
    }

    return null;
  }

  /**
   * Get the value for the input element
   *   @param property - the name of the element
   *   @param className - the name of the class corresponding to this element
   *                      Could be an inner class
   *   @param index - an index for collection
   */
  public Object getElementValue(String property, String className, int index)
  {
    if (isElementQualified()) 
    {
      property = getPrefix() + property;
    }

    NodeList nList = xmlElement.getChildNodes();
    int count = 0;

    for ( int i=0; i < nList.getLength(); i++ )
    {
      Node node = nList.item(i);
      if ( node.getNodeName().equals(property) )
      {
        // ressurect this object
        if (count == index) 
        {
          return factory.newInstance((Element) node, className);
        }
        else
          count++;
      }
    }

    return null;
  }

  /**
   * Update the text node for this element
   */
  public void updateElementValue(String value)
  {
    // If the text node for this element has never been created, create it.
    // Otherwise, update it.
    Node textNode = xmlElement.getFirstChild();
    if (textNode == null)
    {
      factory.createText(xmlElement, value);
    }
    else
    {
      textNode.setNodeValue(value);  
    }
  }

  public void updateElementValue(boolean value)
  {
    updateElementValue((new Boolean(value)).toString());
  }

  public void updateElementValue(int value)
  {
    updateElementValue((new Integer(value)).toString());
  }

  public void updateElementValue(float value)
  {
    updateElementValue((new Float(value)).toString());
  }

  public void updateElementValue(double value)
  {
    updateElementValue((new Double(value)).toString());
  }

  public void updateElementValue(long value)
  {
    updateElementValue((new Long(value)).toString());
  }

  public void updateElementValue(short value)
  {
    updateElementValue((new Short(value)).toString());
  }

  public void updateElementValue(byte value)
  {   
    updateElementValue((new Byte(value)).toString());
  }

  public void updateElementValue(Date value)
  {  
    updateElementValue(simpleDateFormat.format(value));
  }

  /**
   * Remove the element with the input elementName from the current node
   */
  public boolean removeElement(String elementName)
  {
    if (isElementQualified()) 
    {
      elementName = getPrefix() + elementName;
    }

    NodeList nodes = xmlElement.getElementsByTagName(elementName);

    for (int i=0; i < nodes.getLength(); i++)
    {
      Node node = nodes.item(i);
      String item = node.getNodeName().trim();
      if (item.equals(elementName))
      {
        try 
        {
          xmlElement.removeChild(node);
          return true;
        }
        catch (DOMException ex)
        {
        }
      }
    }
    return false;
  }

  /**
   * Return the number of children with this name
   */
  public int sizeOfElement(String name)
  {
    String prefix = "";
    if (isElementQualified()) 
    {
      prefix = getPrefix();
    }
    
    NodeList nl = xmlElement.getChildNodes();
    int nlLength = nl.getLength();
    Node n = null;
    int len = 0;
    String elName = prefix + name;
    for (int i=0; i<nlLength; i++)
    {
      n = nl.item(i);
      if (n.getNodeType() == Node.ELEMENT_NODE
          && ((Element)n).getTagName().equals(elName))
      {
        len++;
      }
    }

    return len;
  }

  /**
   * Return the node name for this element
   */
  public String getElementName()
  {
    return xmlElement.getNodeName();
  }
}  
