/**
 * <copyright>
 *   IBM WebSphere Application Developer XML Tools
 *   (C) Copyright IBM Corp. 2001.
 * </copyright>
 */
package com.ibm.etools.xmlschema.beans;

import org.w3c.dom.*;
import org.xml.sax.InputSource;
import java.util.*;
import java.io.*;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;

/**
 * This factory class creates/loads Java bean and the corresponding DOM node.
 * The generated factory code extends this class.
 */
public class Factory implements java.io.Serializable
{
  protected String xmlFile;
  protected String packageName;
  protected Document document;

  protected String encoding;
  protected String encodingTag;

  protected String dtdFileName;
  protected String dtdPublicId;

  protected String xsdFileName;
  protected String xsdNamespaceURI;
  protected Hashtable importedFileHashtable = new Hashtable();

  protected String rootElementName;

  protected String xmlPrefix = "";
  protected boolean elementQualified = false;
  protected boolean attributeQualified = false;

  public Factory()
  {
  }

  /**
   * Set the name of the Java package
   */
  public void setPackageName(String packageName)
  {
    this.packageName = packageName;
  }

  /**
   * Get the name of the Java package
   */
  public String getPackageName()
  {
    return packageName;
  }

  /**
   * Set the name of the XML instance document
   */
  public void setXMLFilename(String filename)
  {
    this.xmlFile = filename;
  }

  /**
   * Get the name of the XML instance document
   */
  public String getXMLFilename()
  {
    return xmlFile;
  }

  /**
   * Set the encoding that will be used to output the xml document
   *   @param encoding the encoding value for the OutputStreamWriter
   */
  public void setEncoding(String encoding)
  {
    this.encoding = encoding;
  }

  /**
   * Get the encoding that will be used to output the xml document
   */
  public String getEncoding()
  {
    return encoding;
  }

  /**
   * Set the IANA value to be used in the XML declaration
   *  @param encodingTag the encoding tag to be used in the XML declaration
   */
  public void setEncodingTag(String encodingTag)
  {
    this.encodingTag = encodingTag;
  }

  /**
   * Get the IANA value to be used in the XML declaration
   */
  public String getEncodingTag()
  {
    return encodingTag;
  }

  /**
   * Set the DTD file name so that a DOCTYPE will be generated
   * If this is set, the XSD filename should not be set.
   */
  public void setDTDFileName(String dtdFilename)
  {
    this.dtdFileName = dtdFilename;
  }

  /**
   * Get the DTD file name
   */
  public String getDTDFileName()
  {
    return dtdFileName;
  }

  /**
   * Set the public identifier in the DOCTYPE
   */
  public void setPublicId(String publicId)
  {
    this.dtdPublicId = publicId;
  }

  /**
   * Get the public identifier in the DOCTYPE
   */
  public String getPublicId()
  {
    return dtdPublicId;
  }

  /**
   * Set the XSD file name.
   * If this is set, the DTD filename should not be set
   */
  public void setXSDFileName(String xsdFilename)
  {
    this.xsdFileName = xsdFilename;
  }

  /**
   * Get the XSD file name
   */
  public String getXSDFileName()
  {
    return xsdFileName;
  }

  /**
   * Set the namespace URI for this XSD file. 
   */
  public void setNamespaceURI(String namespaceURI)
  {
    xsdNamespaceURI = namespaceURI;
  }

  /**
   * Get the namespace URI
   */
  public String getNamespaceURI()
  {
    return xsdNamespaceURI;
  }

  /**
   * Set the prefix 
   */
  public void setPrefix(String prefix)
  {
    xmlPrefix = (prefix == null) ? "" : prefix + ":";
  }

  /**
   * Get the prefix
   */
  public String getPrefix()
  {
    return xmlPrefix;
  }

  /**
   * Set elementFormQualified for local element
   */
  public void setElementQualified(boolean qualify)
  {
    elementQualified = qualify;
  }

  /**
   * Indicates if local element needs to be qualifed or not
   */
  public boolean isElementQualified()
  {
    return elementQualified;
  }

  /**
   * Set attributeFormQualifed for local attribute
   */
  public void setAttributeQualified(boolean qualify)
  {
    attributeQualified = qualify;
  }

  /**
   * Indicates if local attribute needs to be qualifed or not
   */
  public boolean isAttributeQualified()
  {
    return attributeQualified;
  }

  /**
   * Store the prefix and the namespace information for each imported file
   *  @param filename The filename for the imported schema
   *  @param nsprefix The prefix for the imported schemar
   *  @param nsURI The namespace URI for the imported schema
   */
  public void addImportedFileInfo(String filename, String nsprefix, String nsURI)
  {
    importedFileHashtable.put(nsprefix, nsURI + " " + filename);
  }

  private String getLocalName(Element el)
  {
    String name = el.getTagName();
    if (name.lastIndexOf(":") != -1)
      return name.substring(name.lastIndexOf(":") + 1);
    else
      return name;
  }
  
  private void setRootElementName(String name)
  {
  	if (rootElementName != null)
      return; // already set
      
    rootElementName = name;
  }
  
  /**
   * Load an existing instance document
   */
  public BaseType loadDocument(String classname, String xmlFile)
  {
    this.xmlFile = xmlFile;

    DocumentBuilderFactory docBuilderFactory = DocumentBuilderFactory.newInstance();
    
    try
    {
      DocumentBuilder docBuilder = docBuilderFactory.newDocumentBuilder();
      document = docBuilder.parse(xmlFile);
      Element rootElement = document.getDocumentElement();
      setRootElementName(getLocalName(rootElement));

      return newInstance(rootElement, classname);
    }
    catch (ParserConfigurationException ex)
    {
      System.out.println("JAXP Exception: Factory::loadDocument() " + ex);
    }
    catch (Exception e)
    { 
      System.out.println("Exception: Factory::loadDocument() " + e);
    }
    return null;
  }

  /**
   * Load an existing instance document from an InputSource
   */
  public BaseType loadDocument(String classname, InputSource source)
  {
    DocumentBuilderFactory docBuilderFactory = DocumentBuilderFactory.newInstance();

    try
    {
      DocumentBuilder docBuilder = docBuilderFactory.newDocumentBuilder();
      document = docBuilder.parse(source);
      Element rootElement = document.getDocumentElement();
      setRootElementName(getLocalName(rootElement));

      return newInstance(rootElement, classname);
    }
    catch (ParserConfigurationException ex)
    {
      System.out.println("JAXP Exception: Factory::loadDocument() " + ex);
    }
    catch (Exception e)
    {
      System.out.println("Exception: Factory::loadDocument() " + e);
    }
    return null;
  }

  /**
   * Load an existing instance document from an Element.
   */
  public BaseType loadDocument(String classname, Element element)
  {
    try
    {
      ComplexType domElement = (ComplexType) newInstance(element, classname);
      setRootElementName(getLocalName(element));
      return domElement;
    }
    catch (Exception e)
    {
      System.out.println("Exception: Factory::loadDocument() " + e);
      return null;
    }
  }

  /**
   * Get the document
   */
  public Document getDocument()
  {
    return document;
  }

  /**
   * Save the DOM to the specified file name
   */
  public void save(String filename)
  {
    String outputEncoding = (encoding != null && !encoding.equals(""))
      ? encoding : "UTF8";
    new DOMWriter(document, filename, outputEncoding, encodingTag, 
                  getDocTypeString());
  }

  /**
   * Save the DOM to disk
   */
  public void save()
  {
    if (xmlFile != null) 
    {
      String outputEncoding = (encoding != null && !encoding.equals(""))
        ? encoding : "UTF8";
      new DOMWriter(document, xmlFile, outputEncoding, encodingTag, 
                    getDocTypeString());
    }
  }

  /**
   * Save the DOM to the specified OutputStream
   */
  public void save(OutputStream outStream) 
  {
    String outputEncoding = (encoding != null && !encoding.equals(""))
      ? encoding : "UTF8";

    new DOMWriter(document, outStream, outputEncoding, encodingTag, 
                  getDocTypeString());
  }

  /**
   * Create a new root element from a complex type
   *   @param classname - the name of the Java class that corresponds to the root element
   *   @param rootElementName - the root element name, it might contain a prefix
   *                            e.g. po:purchaseOrder or purchaseOrder
   *   @return ComplexType a Java bean that extends the ComplexType class
   */
  public ComplexType createRootDOMFromComplexType(String classname, String rootElementName)
  {
    return (ComplexType) createRootDOMHelper(classname, rootElementName);
  }

  /**
   * Create a new root element from a simple type
   *   @param classname - the name of the Java class that corresponds to the root element
   *   @param rootElementName - the root element name, it might contain a prefix
   *                            e.g. po:purchaseOrder or purchaseOrder
   *   @return SimpleType a Java bean that extends the SimpleType class
   */
  public SimpleType createRootDOMFromSimpleType(String classname, String rootElementName)
  { 
    return (SimpleType) createRootDOMHelper(classname, rootElementName);
  }

  private Object createRootDOMHelper(String classname, String rootElementName)
  {
    DocumentBuilderFactory docBuilderFactory = DocumentBuilderFactory.newInstance();
    try
    {
      DocumentBuilder docBuilder = docBuilderFactory.newDocumentBuilder();
      document = docBuilder.newDocument();
    }
    catch (ParserConfigurationException ex)
    {
      System.out.println("JAXP Exception: Factory::createRootDOMHelper() " + ex);
      return null;
    }

    this.rootElementName = rootElementName;

    Element childElement = document.createElement(rootElementName);
    document.appendChild(childElement);

    if (xsdFileName != null && !xsdFileName.equals("")) 
    {
      // This XML document is defined by an XSD file. Add the namespace attribute
      childElement.setAttribute("xmlns:xsi",                 
                                "http://www.w3.org/2001/XMLSchema-instance");

      // Set the schemaLocation for validation
      if (xsdNamespaceURI != null && !xsdNamespaceURI.equals("")) 
      {
        int index = rootElementName.indexOf(":");
        if (index > 0) 
        {
          // Add the xmlns attribute for the current XSD file using the prefix
          String prefix = ":" + rootElementName.substring(0,index);
          childElement.setAttribute("xmlns" + prefix, xsdNamespaceURI);
        }
        else
        {
          // Add the xmlns attribute for the current XSD file - without prefix
          childElement.setAttribute("xmlns", xsdNamespaceURI);
        }

        //
        // Add the xmlns attributes for all imported files
        // and compute the validation string to add to the 
        // xsi:schemaLocation attribute
        Enumeration enum = importedFileHashtable.keys();
        String importedFileValidation = "";
        while (enum.hasMoreElements())
        {
          Object nsprefix = enum.nextElement();
          String nsURI = (String) importedFileHashtable.get(nsprefix);
          int blank = nsURI.indexOf(' ');
          childElement.setAttribute("xmlns:" + nsprefix, nsURI.substring(0,blank));
          importedFileValidation += " " + nsURI;
        }

        //
        // Add the xsi:schemaLocation attribute
        //
        childElement.setAttribute("xsi:schemaLocation",
                                  xsdNamespaceURI + " " + xsdFileName + 
                                  importedFileValidation);
      }
      else
      {
        childElement.setAttribute("xsi:noNamespaceSchemaLocation",
                                  xsdFileName);
      }
    }

    return newInstance(childElement, classname);
  }

  /**  
   * Create a new DOM element and the corresponding Java bean. 
   * Called by the generated Factory class.
   *   @param classname The name of the Java bean that corresponds to this element
   *   @param elementName The name of the element
   *   @return ComplexType A Java bean that extends the ComplexType class
   */ 
  public ComplexType createDOMElementFromComplexType(String classname, String elementName)
  {
    String prefix = "";
    if (isElementQualified()) 
    {
      prefix = getPrefix();
    }

    Element childElement = document.createElement(prefix + elementName);
    return (ComplexType) newInstance(childElement, classname);
  }

  /**  
   * Create a new DOM element and the corresponding Java bean. 
   * Called by the generated Factory class.
   *   @param classname The name of the Java bean that corresponds to this element
   *   @param elementName The name of the element
   *   @return SimpleType A Java bean that extends the ComplexType class
   */ 
  public SimpleType createDOMElementFromSimpleType(String classname, String elementName)
  {
    String prefix = "";
    if (isElementQualified()) 
    {
      prefix = getPrefix();
    }

    Element childElement = document.createElement(prefix + elementName);
    return (SimpleType) newInstance(childElement, classname);
  }
   
  /**
   * Create a new Java object for the input Node
   *  @param node - can be an element or an attribute
   *  @param className - the corresponding Java class name
   */
  protected BaseType newInstance(Node node, String className)
  { 
    try
    {       
      String fullname = className;
      if (packageName != null && !packageName.equals(""))
      {
        fullname = packageName + "." + className;
      }

      // Create the Java bean that wraps the DOM node
      Class javaClass = Class.forName(fullname);  
      BaseType object = (BaseType) javaClass.newInstance();

      // Initialize the DOM node in the corresponding Java bean. It can be an 
      // element or an attribute (e.g. in the case when an element's type is a simple type)
      if (node instanceof Element) 
      {
        object.setXMLElement((Element) node);
      }
      else
      {
        object.setXMLAttribute((Attr) node);
      }

      object.setFactory(this);
                                                                            
      return object;
    }
    catch (Exception e)
    {
      System.out.println("Factory::newInstance() error ***" + e);
      return null;
    }
  }

  /**
   * Create a new element and its text node
   *  @param elementName - the name of the new element
   *  @param elementValue - the value of the new element
   */
  protected Element createXMLElementAndText(String elementName, String elementValue)
  {
    String prefix = "";
    if (isElementQualified()) 
    {
      prefix = getPrefix();
    }

    Element item = document.createElement(prefix + elementName);
    
    NodeStringTokenizer tokenizer = new NodeStringTokenizer(elementValue);
    while (tokenizer.hasNextToken())
    {
      NodeString ns = tokenizer.nextToken();
      if (ns.getNodeType() == Node.TEXT_NODE)
      {
        Text text = document.createTextNode(ns.toString());
        item.appendChild(text);
      }
      else
      {
        CDATASection cdata = document.createCDATASection(ns.toString());
        item.appendChild(cdata);
      }
    }
    
    return item;
  }
  
  /**
   * Create a new attribute and add it to the parent element
   */
  protected Attr createAttribute(String name, Element parentElement)
  {
    String prefix = "";
    if (isAttributeQualified()) 
    {
      prefix = getPrefix();
    }

    Attr attrNode = document.createAttribute(prefix + name);
    parentElement.setAttributeNode(attrNode);
    return attrNode;
  }

  /**
   * Create a new Text node and add it to the parent element
   */
  protected Text createText(Element parentElement, String value)
  {
    Text textNode = document.createTextNode(value);
    parentElement.appendChild(textNode);
    return textNode;
  }

  /**
   * Generate a DOCTYPE if the DTD filename is specified
   */
  private String getDocTypeString()
  {
    String docType = "";
    if (dtdFileName != null && !dtdFileName.equals(""))
    {
      if (dtdPublicId == null || dtdPublicId.equals("")) 
      {
      	// defect 238785
        docType = "<!DOCTYPE " + rootElementName + 
                  " SYSTEM \"" + dtdFileName + "\"" +
                  ">";
      }
      else
      {

        docType = "<!DOCTYPE " + rootElementName + 
                  " PUBLIC \"" + dtdPublicId + "\"" + 
                  " \"" + dtdFileName + "\"" +
                  ">";
      }
    }
    return docType;
  }
}
